@@ -51,6 +51,8 @@
 #include "feature/stats/bw_array_st.h"
 #include "feature/relay/router.h"
 
+#include <event2/dns.h>
+
 /** Run unit tests for some stats code. */
 static void
 test_stats(void *arg)
@@ -727,7 +729,7 @@ test_overload_stats(void *arg)
   update_approx_time(current_time);
 
   /* Now check the string */
-  stats_str = rep_hist_get_overload_stats_lines();
+  stats_str = rep_hist_get_overload_general_line();
   tt_str_op("overload-general 1 2002-01-03 23:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
@@ -735,7 +737,7 @@ test_overload_stats(void *arg)
   current_time += 3600*72;
   update_approx_time(current_time);
 
-  stats_str = rep_hist_get_overload_stats_lines();
+  stats_str = rep_hist_get_overload_general_line();
   tt_assert(!stats_str);
 
   /* Now the time should be 2002-01-07 00:00:00 */
@@ -743,16 +745,19 @@ test_overload_stats(void *arg)
   /* Note a DNS overload */
   rep_hist_note_overload(OVERLOAD_GENERAL);
 
-  stats_str = rep_hist_get_overload_stats_lines();
+  stats_str = rep_hist_get_overload_general_line();
   tt_str_op("overload-general 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
   /* Also note an fd exhaustion event */
   rep_hist_note_overload(OVERLOAD_FD_EXHAUSTED);
 
+  stats_str = rep_hist_get_overload_general_line();
+  tt_str_op("overload-general 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
+  tor_free(stats_str);
+
   stats_str = rep_hist_get_overload_stats_lines();
-  tt_str_op("overload-general 1 2002-01-07 00:00:00\n"
-            "overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
+  tt_str_op("overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
   /* Move the time forward. Register DNS overload. See that the time changed */
@@ -761,9 +766,12 @@ test_overload_stats(void *arg)
 
   rep_hist_note_overload(OVERLOAD_GENERAL);
 
+  stats_str = rep_hist_get_overload_general_line();
+  tt_str_op("overload-general 1 2002-01-07 02:00:00\n", OP_EQ, stats_str);
+  tor_free(stats_str);
+
   stats_str = rep_hist_get_overload_stats_lines();
-  tt_str_op("overload-general 1 2002-01-07 02:00:00\n"
-            "overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
+  tt_str_op("overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
   /* Move the time forward. Register a bandwidth ratelimit event. See that the
@@ -777,9 +785,12 @@ test_overload_stats(void *arg)
   get_options_mutable()->BandwidthRate = 1000;
   get_options_mutable()->BandwidthBurst = 2000;
 
+  stats_str = rep_hist_get_overload_general_line();
+  tt_str_op("overload-general 1 2002-01-07 02:00:00\n", OP_EQ, stats_str);
+  tor_free(stats_str);
+
   stats_str = rep_hist_get_overload_stats_lines();
-  tt_str_op("overload-general 1 2002-01-07 02:00:00\n"
-            "overload-ratelimits 1 2002-01-07 04:00:00 1000 2000 1 0\n"
+  tt_str_op("overload-ratelimits 1 2002-01-07 04:00:00 1000 2000 1 0\n"
             "overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
@@ -787,16 +798,19 @@ test_overload_stats(void *arg)
   current_time += 3600*24;
   update_approx_time(current_time);
 
+  stats_str = rep_hist_get_overload_general_line();
+  tt_str_op("overload-general 1 2002-01-07 02:00:00\n", OP_EQ, stats_str);
+  tor_free(stats_str);
+
   stats_str = rep_hist_get_overload_stats_lines();
-  tt_str_op("overload-general 1 2002-01-07 02:00:00\n"
-            "overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
+  tt_str_op("overload-fd-exhausted 1 2002-01-07 00:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
   /* Move the time forward 44 hours: no fd exhausted line anymore. */
   current_time += 3600*44;
   update_approx_time(current_time);
 
-  stats_str = rep_hist_get_overload_stats_lines();
+  stats_str = rep_hist_get_overload_general_line();
   tt_str_op("overload-general 1 2002-01-07 02:00:00\n", OP_EQ, stats_str);
   tor_free(stats_str);
 
@@ -804,6 +818,9 @@ test_overload_stats(void *arg)
   current_time += 3600*2;
   update_approx_time(current_time);
 
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(!stats_str);
+
   stats_str = rep_hist_get_overload_stats_lines();
   tt_assert(!stats_str);
 
@@ -850,6 +867,81 @@ test_overload_stats(void *arg)
   tor_free(stats_str);
 }
 
+/** Test the overload stats logic. */
+static void
+test_overload_dns_timeout(void *arg)
+{
+  char *stats_str = NULL;
+  (void) arg;
+
+  /* Lets simulate a series of timeouts but below our default 1% threshold. */
+
+  for (int i = 0; i < 1000; i++) {
+    /* This should trigger 9 timeouts which is just below 1% (10) */
+    if (i > 0 && !(i % 100)) {
+      rep_hist_note_dns_error(0, DNS_ERR_TIMEOUT);
+    } else {
+      rep_hist_note_dns_error(0, DNS_ERR_NONE);
+    }
+  }
+
+  /* No overload yet. */
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(!stats_str);
+
+  /* Move it 10 minutes in the future and see if we get a general overload. */
+  update_approx_time(approx_time() + (10 * 60));
+
+  /* This query should NOT trigger the general overload because we are below
+   * our default of 1%. */
+  rep_hist_note_dns_error(0, DNS_ERR_NONE);
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(!stats_str);
+
+  /* We'll now go above our 1% threshold. */
+  for (int i = 0; i < 1000; i++) {
+    /* This should trigger 10 timeouts which is our threshold of 1% (10) */
+    if (!(i % 10)) {
+      rep_hist_note_dns_error(0, DNS_ERR_TIMEOUT);
+    } else {
+      rep_hist_note_dns_error(0, DNS_ERR_NONE);
+    }
+  }
+
+  /* Move it 10 minutes in the future and see if we get a general overload. */
+  update_approx_time(approx_time() + (10 * 60));
+
+  /* This query should trigger the general overload because we are above 1%. */
+  rep_hist_note_dns_error(0, DNS_ERR_NONE);
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(stats_str);
+  tor_free(stats_str);
+
+  /* Move 72h in the future, we should NOT get an overload anymore. */
+  update_approx_time(approx_time() + (72 * 3600));
+
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(!stats_str);
+
+  /* This query should NOT trigger the general overload. */
+  rep_hist_note_dns_error(0, DNS_ERR_TIMEOUT);
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(!stats_str);
+
+  /* Move it 10 minutes in the future and see if we get a general overload. We
+   * have now 100% of requests timing out. */
+  update_approx_time(approx_time() + (10 * 60));
+
+  /* This query should trigger the general overload with 50% of timeouts. */
+  rep_hist_note_dns_error(0, DNS_ERR_NONE);
+  stats_str = rep_hist_get_overload_general_line();
+  tt_assert(stats_str);
+  tor_free(stats_str);
+
+ done:
+  tor_free(stats_str);
+}
+
 #define ENT(name)                                                       \
   { #name, test_ ## name , 0, NULL, NULL }
 #define FORK(name)                                                      \
@@ -866,6 +958,7 @@ struct testcase_t stats_tests[] = {
   FORK(rephist_v3_onions),
   FORK(load_stats_file),
   FORK(overload_stats),
+  FORK(overload_dns_timeout),
 
   END_OF_TESTCASES
 };
